//******************************************************************************
// FILE:       	lcd.h
//              
// AUTHOR:     	CC Cox - SynchroSystems
//              
// PROJECT:    	SynchroChron
//              
// DESCRIPTION:	LCD driver interface file
//
// HISTORY:
//		$Log: lcd.h,v $
//		Revision 1.2  2006-01-26 16:32:54-05  chuck
//		Release candidate
//
//		Revision 1.1 2006-01-13 chuck
//		Code review changes
//
//		Revision 1.0 2005-03-11 chuck
//		Initial revision
//
// Copyright (c) 2006 SynchroSystems Inc
// Use subject to terms of License Agreement at http://synchro.com/policies.html
//******************************************************************************

#ifndef __LCD_H
#define __LCD_H

//******************************************************************************
// PUBLIC DECLARATIONS
//******************************************************************************

//------------------------------------------------------------------------------
// constants
//------------------------------------------------------------------------------

// flash flags for display functions
#define STEADY				FALSE		// flashing disabled
#define FLASH				TRUE		// flashing enabled
// leading-zero flags for integer functions
#define LEAD_SPACE			FALSE		// show leading spaces
#define LEAD_ZERO			TRUE		// show leading zeros
// display area sizes
#define LCD_PRIM_LEN		6			// character cells in primary area
#define LCD_SEC_LEN			4			// digit cells in secondary area
#define LCD_MODE_LEN		2			// character cells in mode area
#define LCD_PRIM_PUNCT_LEN	2			// punctuation cells in primary area
#define LCD_SEC_PUNCT_LEN	1			// punctuation cells in secondary area
#define LCD_ICON_LEN		2			// icons
// maximum segment masks
#define LCD_ALPHA_ALL_SEGS	0x3FFFu		// all segments in alphanumeric cell
#define LCD_NUMER_ALL_SEGS	0x007Fu		// all segments in numeric cell
#define LCD_PUNCT_ALL_SEGS	0x0003u		// all segments in punctuation cell
#define LCD_ICON_ALL_SEGS	0x0001u		// all segments in icons
#define LCD_DIAL_ALL_SEGS	0x0FFFu		// all segments in dial
// icon segments
#define LCD_ICON_OFF		0x0000u		// icon segment off
#define LCD_ICON_ON			0x0001u		// icon segment on
#define LCD_ALARM_POS		0			// alarm icon position
#define LCD_BELL_POS		1			// bell icon position
#define LCD_HAND(x)			(1 << (x))	// dial hand segment for 0..11
// dial hand mask lengths
#define LCD_DIAL_12_LEN		12			// 12 hours / half day
#define LCD_DIAL_60_LEN		60			// 60 minutes/hour or seconds/minute
#define LCD_DIAL_100_LEN	100			// 100 ticks/second
// segment bit masks
#define SEGA 	0x0001u
#define SEGB 	0x0002u
#define SEGC 	0x0004u
#define SEGD 	0x0008u
#define SEGE 	0x0010u
#define SEGF 	0x0020u
#define SEGG 	0x0040u
#define SEGH 	0x0080u
#define SEGJ 	0x0100u
#define SEGK 	0x0200u
#define SEGM 	0x0400u
#define SEGN 	0x0800u
#define SEGP 	0x1000u
#define SEGQ 	0x2000u

//------------------------------------------------------------------------------
// macros
//------------------------------------------------------------------------------
#define H12(x)	(1 << (x))					// dial hand mask for 0..11
#define H60(x)	(1 << ((12 * (x)) / 60))	// dial hand mask for 0..59
#define H100(x)	(1 << ((12 * (x)) / 100))	// dial hand mask for 0..99

//------------------------------------------------------------------------------
//  MACRO:  		LcdShowDial12
//              
//  PARAMETERS: 	value		value to represent (0 - 23)
//					flash		flash enable flag
//
//	RETURNS:		nothing
//              
//  DESCRIPTION:	Display a dial hand representing the value 0..23 modulus 12
//------------------------------------------------------------------------------
#define LcdShowDial12(value, flash)		\
	LcdShowSegs(LCD_DIAL_12[(value) < 12 ? (value) : (value) - 12], LCD_DIAL_AREA, 0, (BOOL)(flash))
	
//------------------------------------------------------------------------------
//  MACRO:   	    LcdShowDial60
//              
//  PARAMETERS: 	value		value to represent (0 - 59)
//					flash		flash enable flag
//              
//	RETURNS:		nothing
//              
//  DESCRIPTION:	Display a dial hand representing the value 0..59
//------------------------------------------------------------------------------
#define LcdShowDial60(value, flash) 	\
	LcdShowSegs(LCD_DIAL_60[(value)], LCD_DIAL_AREA, 0, (BOOL)(flash))
	
//------------------------------------------------------------------------------
//  MACRO:   	    LcdShowDial100
//              
//  PARAMETERS: 	value		value to represent (0 - 99)
//					flash		flash enable flag
//              
//	RETURNS:		nothing
//              
//  DESCRIPTION:	Display a dial hand representing the value 0..99
//------------------------------------------------------------------------------
#define LcdShowDial100(value, flash) 	\
	LcdShowSegs(LCD_DIAL_100[(value)], LCD_DIAL_AREA, 0, (BOOL)(flash))
	
//------------------------------------------------------------------------------
//  MACRO:   	    LcdShowDial
//              
//  PARAMETERS: 	mask	    dial segment mask
//  			 	flash		flash enable flag
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	Display dial segments
//------------------------------------------------------------------------------
#define LcdShowDial(mask, flash) \
	LcdShowSegs(mask, LCD_DIAL_AREA, 0, flash);

//------------------------------------------------------------------------------
// typedefs
//------------------------------------------------------------------------------

//
// LCD_SEGS index offsets
//
// offsets into the LCD_SEGS array for each display area
//
typedef enum
{
	LCD_MODE_AREA			= 0,	// 2 mode characters
	LCD_SEC_AREA			= 2,	// 4 secondary digits
	LCD_PRIM_AREA			= 6,	// 6 primary characters
	LCD_SEC_PUNCT_AREA		= 12,	// 1 secondary punctuation mark
	LCD_PRIM_PUNCT_AREA		= 13,	// 2 primary punctuation marks
	LCD_DIAL_AREA			= 15,	// 1 dial
	LCD_ICON_AREA			= 16	// 2 icons
} LCD_AREA;

//------------------------------------------------------------------------------
// ROM data
//------------------------------------------------------------------------------
extern const UINT16 LCD_DIAL_12[LCD_DIAL_12_LEN];
extern const UINT16 LCD_DIAL_60[LCD_DIAL_60_LEN];
extern const UINT16 LCD_DIAL_100[LCD_DIAL_100_LEN];

//------------------------------------------------------------------------------
// functions
//------------------------------------------------------------------------------
//
// control functions
//
extern void LcdInit(void);
extern void LcdOn(void);
extern void LcdOff(void);
extern void LcdClear(void);
extern void LcdFlashSuppress(UINT8 duration);
extern void LcdFlashTimer(void);
extern void LcdScrollTimer(void);
extern BOOL LcdIsScrolling(void);
//
// display functions
//
extern void LcdShowSegs(UINT16 segs, LCD_AREA area, UINT8 pos, BOOL flash);
extern void LcdShowChar(UINT8 c, LCD_AREA area, UINT8 pos, BOOL flash);
extern void LcdShowStr(const UINT8* s, LCD_AREA area, UINT8 pos, BOOL flash);
extern void LcdShowStrN(const UINT8* s, LCD_AREA area, UINT8 pos, UINT8 len, BOOL flash);
extern void LcdShowScroll(const UINT8* s);
extern void LcdShowUDec2(UINT8 value, LCD_AREA area, UINT8 pos, BOOL flash, BOOL lead_zero);
extern void LcdShowUDec4(UINT16 value, LCD_AREA area, UINT8 pos, BOOL flash, BOOL lead_zero);
extern void LcdShowDialCW(UINT16* mask_ptr, BOOL flash);
extern void LcdShowDialCCW(UINT16* mask_ptr, BOOL flash);
extern void LcdShowError(const UINT8* mode_ptr, const UINT8* str_ptr, UINT16 num);
//
// stdio functions
//
extern int putchar(PUTCHAR_ARG_TYPE c);

#endif	// __LCD_H
